@@ -1,12 +1,11 @@
 #'@title Compute Measures for Identifying Patterns of Change in Longitudinal
 #'  Data
 #'
-#'@description \code{Step1Measures} computes up to 26 measures for each
+#'@description \code{Step1Measures} computes up to 18 measures for each
 #'  longitudinal trajectory. See Details for the list of measures.
-#'
 #'@param Data a matrix or data frame in which each row contains the longitudinal
 #'  data (trajectories).
-#'@param Time either NULL, a vector or a matrix/data frame of the same dimension
+#'@param Time either \code{NULL}, a vector or a matrix/data frame of the same dimension
 #'  as \code{Data}. If a vector, matrix or data frame is supplied, its entries
 #'  are assumed to be measured at the times of the corresponding cells in
 #'  \code{Data}. When set to \code{NULL} (the default), the times are assumed
@@ -15,14 +14,15 @@
 #'  \code{Time} corresponds to an \code{ID} variable identifying the
 #'  trajectories. Defaults to \code{FALSE}.
 #'@param measures a vector containing the numerical identifiers of the measures
-#'  to compute (see "Details" section below). The default, 1:23, corresponds to
-#'  measures 1-23 and thus excludes the measures which require specifying a
+#'  to compute (see "Details" section below). The default, 1:17, corresponds to
+#'  measures 1-17 and thus excludes the measures which require specifying a
 #'  midpoint.
 #'@param midpoint specifies which column of \code{Time} to use as the midpoint
-#'  in measures 24-26. Can be NULL, an integer or a vector of integers of length
-#'  the number of rows in \code{Time}. The default is NULL, in which case the
+#'  in measure 18. Can be \code{NULL}, an integer or a vector of integers of length
+#'  the number of rows in \code{Time}. The default is \code{NULL}, in which case the
 #'  midpoint is the time closest to the median of the Time vector specific to
 #'  each trajectory.
+#'@param cap.outliers logical. If \code{TRUE}, extreme values of the measures will be caped (see details below). Defaults to \code{TRUE}.
 #'@param x object of class trajMeasures.
 #'@param object object of class trajMeasures.
 #'@param ... further arguments passed to or from other methods.
@@ -34,37 +34,41 @@
 #'@details Each trajectory must have a minimum of 3 observations otherwise it
 #'  will be omitted from the analysis.
 #'
-#'  The 26 measures and their numerical identifiers are listed below. Please
+#'  The 18 measures and their numerical identifiers are listed below. Please
 #'  refer to the vignette for the specific formulas used to compute them.
 #'\enumerate{
-#'\item  Range\cr
-#'\item  Mean of the function\cr
-#'\item  Functional standard deviation (SD)\cr
-#'\item  Coefficient of variation (ratio of measure 3 to measure 2)\cr
-#'\item  Overall change (initial value - final value)\cr
-#'\item  Mean change per unit time\cr
-#'\item  Overall change relative to initial value\cr
-#'\item  Overall change relative to functional mean (ratio of measure 5 to measure 2)\cr
+#'\item  Maximum\cr
+#'\item  Range (max - min)\cr
+#'\item  Mean value\cr
+#'\item  Standard deviation\cr
 #'\item  Slope of the linear model\cr
 #'\item  \eqn{R^2}: Proportion of variance explained by the linear model\cr
-#'\item  Maximum value of the speed\cr
-#'\item  Functional SD of the speed\cr
-#'\item  Mean absolute speed\cr
-#'\item  Maximum absolute speed\cr
-#'\item  Maximum absolute speed relative to the functional mean (ratio of measure 14 to measure 2)\cr
-#'\item  Maximum absolute speed relative to the slope (ratio of measure 14 to measure 9)\cr
-#'\item  Functional SD of the speed relative to the slope\cr(ratio of measure 12 to measure 9)
-#'\item  Mean acceleration\cr
-#'\item  Mean absolute acceleration\cr
-#'\item  Maximum of the absolute acceleration\cr
-#'\item  Maximum of the absolute acceleration relative to the functional (ratio of measure 20 to measure 2)\cr
-#'\item  Maximum of the absolute acceleration relative to the mean absolute speed (ratio of measure 20 to measure 13)\cr
-#'\item  Mean absolute acceleration relative to the mean absolute speed (ratio of measure 19 to measure 13)\cr
-#'\item  Early change relative to later change\cr
-#'\item  Early change relative to overall change\cr
-#'\item  Later change relative to overall change\cr
+#'\item  Curve length (total variation)\cr
+#'\item  Rate of intersection with the mean\cr
+#'\item  Proportion of time spent under the mean\cr
+#'\item  Minimum  of the first derivative\cr
+#'\item  Maximum of the first derivative\cr
+#'\item  Mean of the first derivative\cr
+#'\item  Standard deviation of the first derivative\cr
+#'\item  Minimum  of the second derivative\cr
+#'\item  Maximum of the second derivative\cr
+#'\item  Mean of the second derivative\cr
+#'\item  Standard deviation of the second derivative\cr
+#'\item  Early change/Later change\cr
 #'}
 #'
+#'  In the presence of highly correlated measures (Pearson correlation >
+#'  0.98), the function selects the highest-ranking measure on the list (see
+#'  \code{\link[traj]{Step1Measures}}) and discards the others. Because the
+#'  K-means algorithm is sensitive to outliers, the measures are prevented from taking extreme or
+#'  infinite values (caused by a possible division by 0 in m18). Nishiyama's improved Chebychev
+#'  bound is used to determine extreme values for each measure, corresponding to
+#'  a 0.3% probability threshold. Extreme values beyond the threshold are then capped
+#'  to the 0.3% probability threshold. If applicable, the values of m18 which
+#'  would be of the form 0/0 are set to 1. PCA is applied on the remaining
+#'  measures using the \code{\link[psych]{principal}} function from the
+#'  \code{psych} package.
+#'
 #'@importFrom stats complete.cases coefficients lm median sd density
 #'
 #'@references Leffondre K, Abrahamowicz M, Regeasse A, Hawker GA, Badley EM,
@@ -79,10 +83,13 @@
 #'
 #'@examples
 #'\dontrun{
-#'m1 = Step1Measures(trajdata, ID = TRUE, measures = 24:26, midpoint = NULL)
-#'m2 = Step1Measures(trajdata, ID = TRUE, measures = 24:26, midpoint = 3)
+#'data("trajdata")
+#'trajdata.noGrp <- trajdata[, which(colnames(trajdata) == "Group")] #remove the Group column
 #'
-#'identical(s1$measures, s2$measures)
+#'m1 = Step1Measures(trajdata.noGrp, ID = TRUE, measures = 18, midpoint = NULL)
+#'m2 = Step1Measures(trajdata.noGrp, ID = TRUE, measures = 18, midpoint = 3)
+#'
+#'identical(m1$measures, m2$measures)
 #'}
 #'
 #'@rdname Step1Measures
@@ -92,8 +99,10 @@ Step1Measures <-
   function (Data,
             Time = NULL,
             ID = FALSE,
-            measures = 1:23,
-            midpoint = NULL) {
+            measures = c(1:17),
+            midpoint = NULL,
+            cap.outliers = TRUE) {
+    
     ###############################################################
     ##         Perform checks and uniformization of data         ##
     ###############################################################
@@ -258,14 +267,14 @@ Step1Measures <-
     ##         Construct vector of "mid points"         ##
     ######################################################
     
-    if (TRUE %in% (c(25:27) %in% measures)) {
+    if (TRUE %in% (18 %in% measures)) {
       mid.position <- c()
       
       if (is.null(midpoint)) {
         median.time <- (max(time, na.rm = TRUE) + min(time, na.rm = TRUE)) / 2
         flag <- c()
         for (i in seq_len(nrow(data))) {
-          v <- time[i,][!is.na(time[i,])]
+          v <- time[i, ][!is.na(time[i, ])]
           w <- which.min(abs(v - median.time))
           mid.position[i] <- w
           if (w == length(v) | w == 1) {
@@ -275,8 +284,8 @@ Step1Measures <-
         }
         if (length(flag) > 0) {
           mid.position <- mid.position[-flag]
-          data <- data[-flag,]
-          time <- time[-flag,]
+          data <- data[-flag, ]
+          time <- time[-flag, ]
           if (ID == TRUE) {
             Lines <- which(Data[, 1] %in% IDvector[flag])
           } else{
@@ -287,7 +296,7 @@ Step1Measures <-
               paste(
                 "When left blank, the 'midpoint' argument defaults to the observation time closests to the median time 0.5*(max(Time)-min(Time)), but this can't be either the first or last observation time. As a result, row ",
                 Lines,
-                " has been removed. To avoid this, consider excluding measures 24-26 from the analysis or providing custom 'midpoint' values.",
+                " has been removed. To avoid this, consider excluding measure 18 from the analysis or providing custom 'midpoint' values.",
                 sep = ""
               )
             )
@@ -297,7 +306,7 @@ Step1Measures <-
               paste(
                 "When left blank, the 'midpoint' argument defaults to the observation time closests to the median time 0.5*(max(Time)-min(Time)), but this can't be either the first or last observation time. As a result, rows ",
                 noquote(paste(Lines, collapse = ", ")),
-                " have been removed. To avoid this, consider excluding measures 24-26 from the analysis or providing custom 'midpoint' values.",
+                " have been removed. To avoid this, consider excluding measure 18 from the analysis or providing custom 'midpoint' values.",
                 sep = ""
               )
             )
@@ -322,7 +331,7 @@ Step1Measures <-
       }
       # Check to see if the midpoints are all greater than 1 but less than the number of observations
       for (i in seq_len(nrow(data))) {
-        v <- time[i,][!is.na(time[i,])]
+        v <- time[i,][!is.na(time[i, ])]
         if (mid.position[i] <= 1) {
           stop(
             paste(
@@ -347,9 +356,9 @@ Step1Measures <-
       mid.position <- NULL
     }
     
-    #################################################################################u########
+    ##########################################################################################
     ##         Initialize main output data frame and compute the requested measures         ##
-    ################################################################################u#########
+    ##########################################################################################
     
     output <-
       data.frame(matrix(ncol = 1 + length(measures), nrow = nrow(data)))
@@ -359,79 +368,50 @@ Step1Measures <-
     data <- as.matrix(data)
     time <- as.matrix(time)
     
+    
+    #max
     if (1 %in% measures) {
       for (i in seq_len(nrow(data))) {
         output$m1[i] <-
-          max(data[i,], na.rm = TRUE) - min(data[i,], na.rm = TRUE)
+          max(data[i, ], na.rm = TRUE)
       }
     }
     
-    if (sum(c(2, 3, 4, 8, 15, 21) %in% measures) > 0) {
-      m2 <- c()
+    # range 
+    if (2 %in% measures) {
       for (i in seq_len(nrow(data))) {
-        y <- data[i, complete.cases(data[i,])]
-        x <- time[i, complete.cases(time[i,])]
-        m2[i] <- FctMean(x, y)
-      }
-      if (2 %in% measures) {
-        output$m2 <- m2
+        output$m2[i] <-
+          max(data[i, ], na.rm = TRUE) - min(data[i, ], na.rm = TRUE)
       }
     }
     
-    if (sum(c(3, 4) %in% measures) > 0) {
+    #mean
+    if (sum(c(3, 4, 8, 9) %in% measures) > 0) {
       m3 <- c()
       for (i in seq_len(nrow(data))) {
-        y <- data[i, complete.cases(data[i,])]
-        x <- time[i, complete.cases(time[i,])]
-        m3[i] <- sqrt(FctMean(x, (y - m2[i]) ^ 2))
+        y <- data[i, complete.cases(data[i, ])]
+        x <- time[i, complete.cases(time[i, ])]
+        m3[i] <- FctMean(x, y)
       }
       if (3 %in% measures) {
         output$m3 <- m3
       }
     }
     
+    #SD
     if (4 %in% measures) {
       for (i in seq_len(nrow(data))) {
-        output$m4[i] <- output$m3[i] / output$m2[i]
+        y <- data[i, complete.cases(data[i, ])]
+        x <- time[i, complete.cases(time[i, ])]
+        output$m4[i] <- sqrt(FctMean(x, (y - m3[i]) ^ 2))
       }
     }
     
-    if (sum(c(5, 6, 7, 8, 25, 26) %in% measures) > 0) {
-      m5 <- c()
+    #slope of linear model
+    if (5 %in% measures) {
       for (i in seq_len(nrow(data))) {
-        m5[i] <- Last(data[i,]) - First(data[i,])
-      }
-      if (5 %in% measures) {
-        output$m5 <- m5
-      }
-    }
-    
-    if (6 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m6[i] <- m5[i] / (Last(time[i,]) - First(time[i,]))
-      }
-    }
-    
-    if (7 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m7[i] <- m5[i] / First(data[i,])
-      }
-    }
-    
-    if (8 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m8[i] <- m5[i] / m2[i]
-      }
-    }
-    
-    if (sum(c(9, 16, 17) %in% measures) > 0) {
-      m9 <- c()
-      for (i in seq_len(nrow(data))) {
-        b <- coefficients(lm(data[i,] ~ time[i,]))
-        m9[i] <- b[2]
-      }
-      if (9 %in% measures) {
-        output$m9 <- m9
+        b <- coefficients(lm(data[i, ] ~ time[i, ]))
+        output$m5[i] <- b[2]
       }
     }
     
@@ -439,279 +419,296 @@ Step1Measures <-
     defaultW <- getOption("warn")
     options(warn = -1)
     
-    if (10 %in% measures) {
+    #R^2
+    if (6 %in% measures) {
       for (i in seq_len(nrow(data))) {
-        model <- lm(data[i,] ~ time[i,])
-        output$m10[i] <- summary(model)$r.squared
+        model <- lm(data[i, ] ~ time[i, ])
+        output$m6[i] <- summary(model)$r.squared
       }
     }
-    
     options(warn = defaultW)
     
-    if (11 %in% measures) {
+    #curve length:
+    if (7 %in% measures) { 
       for (i in seq_len(nrow(data))) {
-        y <- data[i, complete.cases(data[i,])]
-        x <- time[i, complete.cases(time[i,])]
-        D <- Der(x, y)
-        output$m11[i] <- max(D)
+        y <- data[i, complete.cases(data[i, ])]
+        x <- time[i, complete.cases(time[i, ])]
+        Length <- 0
+        for(k in seq_along(y)[-length(y)]){
+          Length <- Length + sqrt( (x[k+1] - x[k])^2 + (y[k+1] - y[k])^2 )
+        }
+        output$m7[i] <- Length
       }
     }
     
-    if (sum(c(12, 17) %in% measures) > 0) {
-      m12 <- c()
+    #rate of intersections with the mean
+    
+    if (8 %in% measures) { 
+      intersection.count <- c()
       for (i in seq_len(nrow(data))) {
-        y <- data[i, complete.cases(data[i,])]
-        x <- time[i, complete.cases(time[i,])]
-        D <- Der(x, y)
-        m12[i] <- sqrt(FctMean(x, (D - FctMean(x, D)) ^ 2))
+        norm <- data[i, ] - m3[i]
+        y <- norm[complete.cases(norm)]
+        x <- time[i, complete.cases(time[i, ])]
+        
+        intersection.count[i] <- 0
+        for(k in  seq_along(y)[-length(y)]){
+          w <- which(sign(y[k] * y[(k+1):length(y)]) !=0)
+          if(length(w) > 0){
+            if(sign(y[k] * y[(k+1):length(y)])[w[1]] == -1){
+              intersection.count[i] <- intersection.count[i] + 1 
+            }
+          }
+        }
+        output$m8[i] <- intersection.count[i] /( max(x) - min(x) )
+      }
+    }
+
+    #fraction of time spent under the mean (if this is large, then there are big sharp spikes)
+    if (9 %in% measures) {
+      for (i in seq_len(nrow(data))) {
+        norm <- data[i, ] - m3[i]
+        y <- norm[complete.cases(norm)]
+        x <- time[i, complete.cases(time[i, ])]
+        
+        blue.time <- 0
+        wb <- which(y > 0)
+        if(length(wb > 0)){
+          for (k in wb) {
+            if(k == 1){
+              blue.time <- blue.time + 0.5*(x[k+1] - x[k])
+            }
+            if (! (k %in% c(1, length(x)))) {
+              blue.time <- blue.time + 0.5*(x[k+1] - x[k-1])
+            }
+            if(k == length(x)){
+              blue.time <- blue.time + 0.5*(x[k] - x[k-1])
+            }
+          }
+        }
+        
+        red.time <- 0
+        wr <- which(y < 0)
+        if(length(wr > 0)){
+          for (k in wr) {
+            if(k == 1){
+              red.time <- red.time + 0.5*(x[k+1] - x[k])
+            }
+            if (! (k %in% c(1, length(x)))) {
+              red.time <- red.time + 0.5*(x[k+1] - x[k-1])
+            }
+            if(k == length(x)){
+              red.time <- red.time + 0.5*(x[k] - x[k-1])
+            }
+          }
+        }
+        output$m9[i] <- blue.time/(blue.time + red.time)
+      }
+    }
+    
+    
+    ### Measures on y'(t) ###
+    
+    # If a measure involving the speed or the acceleration was requested, compute the derivative:
+    if (sum(measures %in% 10:17) > 0) {
+      speed.data <- data
+      for (i in seq_len(nrow(data))) {
+        y <- data[i, complete.cases(data[i, ])]
+        x <- time[i, complete.cases(time[i, ])]
+        speed.data[i, complete.cases(speed.data[i, ])] <- Der(x, y)
+      }
+    }
+    
+    #min
+    if (10 %in% measures) {
+      for (i in seq_len(nrow(speed.data))) {
+        output$m10[i] <-
+          min(speed.data[i, ], na.rm = TRUE)
+      }
+    }
+    
+    #max
+    if (11 %in% measures) {
+      for (i in seq_len(nrow(speed.data))) {
+        output$m11[i] <-
+          max(speed.data[i, ], na.rm = TRUE)
+      }
+    }
+    
+    #mean 
+    if (sum(c(12, 13) %in% measures) > 0) {
+      m12 <- c()
+      for (i in seq_len(nrow(speed.data))) {
+        y <- speed.data[i, complete.cases(speed.data[i, ])]
+        x <- time[i, complete.cases(time[i, ])]
+        m12[i] <- FctMean(x, y)
       }
       if (12 %in% measures) {
         output$m12 <- m12
       }
     }
     
-    if (sum(c(13, 22, 23) %in% measures) > 0) {
-      m13 <- c()
-      for (i in seq_len(nrow(data))) {
-        y <- data[i, complete.cases(data[i,])]
-        x <- time[i, complete.cases(time[i,])]
-        D <- Der(x, y)
-        m13[i] <- FctMean(x, abs(D))
-      }
-      if (13 %in% measures) {
-        output$m13 <- m13
+    #SD
+    if (13 %in% measures) {
+      for (i in seq_len(nrow(speed.data))) {
+        y <- speed.data[i, complete.cases(speed.data[i, ])]
+        x <- time[i, complete.cases(time[i, ])]
+        output$m13[i] <- sqrt(FctMean(x, (y - m12[i]) ^ 2))
       }
     }
     
-    if (sum(c(14, 15, 16) %in% measures) > 0) {
-      m14 <- c()
-      for (i in seq_len(nrow(data))) {
-        y <- data[i, complete.cases(data[i,])]
-        x <- time[i, complete.cases(time[i,])]
-        D <- Der(x, y)
-        m14[i] <- max(abs(D))
-      }
-      if (14 %in% measures) {
-        output$m14 <- m14
+    ### Measures on y''(t) ###
+    
+    #if a measure involving the acceleration was requested, compute the derivative of the derivative:
+    
+    
+    
+    if (sum(measures %in% 14:17) > 0) {
+      accel.data <- speed.data
+      for (i in seq_len(nrow(accel.data))) {
+        y <- speed.data[i, complete.cases(speed.data[i, ])]
+        x <- time[i, complete.cases(time[i, ])]
+        accel.data[i, complete.cases(accel.data[i, ])] <- Der(x, y)
       }
     }
     
+    #min
+    if (14 %in% measures) {
+      for (i in seq_len(nrow(accel.data))) {
+        output$m14[i] <-
+          min(accel.data[i, ], na.rm = TRUE)
+      }
+    }
+    
+    #max
     if (15 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m15[i] <- m14[i] / m2[i]
+      for (i in seq_len(nrow(accel.data))) {
+        output$m15[i] <-
+          max(accel.data[i, ], na.rm = TRUE)
       }
     }
     
-    if (16 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m16[i] <- m14[i] / m9[i]
+    #mean 
+    if (sum(c(16, 17) %in% measures) > 0) {
+      m16 <- c()
+      for (i in seq_len(nrow(accel.data))) {
+        y <- accel.data[i, complete.cases(accel.data[i, ])]
+        x <- time[i, complete.cases(time[i, ])]
+        m16[i] <- FctMean(x, y)
+      }
+      if (16 %in% measures) {
+        output$m16 <- m16
       }
     }
     
+    #SD
     if (17 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m17[i] <- m12[i] / m9[i]
+      for (i in seq_len(nrow(accel.data))) {
+        y <- accel.data[i, complete.cases(accel.data[i, ])]
+        x <- time[i, complete.cases(time[i, ])]
+        output$m17[i] <- sqrt(FctMean(x, (y - m16[i]) ^ 2))
       }
     }
     
+    #later change/early change
     if (18 %in% measures) {
       for (i in seq_len(nrow(data))) {
-        y <- data[i, complete.cases(data[i,])]
-        x.i <- time[i, complete.cases(time[i,])]
-        D <- Der(x.i, y)
-        A <- Der(x.i, D)
-        output$m18[i] <- FctMean(x.i, A)
+        early <- Last(data[i, 1:mid.position[i]]) - First(data[i, 1:mid.position[i]])
+        later <- Last(data[i, mid.position[i]:ncol(data)]) - First(data[i, mid.position[i]:ncol(data)])
+        output$m18[i] <- later/early
       }
     }
+
+    output[is.na(output)] <- 1 #  NAs correspond to quotient measures of the form 0/0
     
-    if (sum(c(19, 23) %in% measures) > 0) {
-      m19 <- c()
-      for (i in seq_len(nrow(data))) {
-        y <- data[i, complete.cases(data[i,])]
-        x.i <- time[i, complete.cases(time[i,])]
-        D <- Der(x.i, y)
-        A <- Der(x.i, D)
-        m19[i] <- FctMean(x.i, abs(A))
-      }
-      if (19 %in% measures) {
-        output$m19 <- m19
-      }
-    }
-    
-    if (sum(c(20, 21, 22) %in% measures) > 0) {
-      m20 <- c()
-      for (i in seq_len(nrow(data))) {
-        y <- data[i, complete.cases(data[i,])]
-        x.i <- time[i, complete.cases(time[i,])]
-        D <- Der(x.i, y)
-        A <- Der(x.i, D)
-        m20[i] <- max(abs(A))
-      }
-      if (20 %in% measures) {
-        output$m20 <- m20
-      }
-    }
-    
-    if (21 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m21[i] <- m20[i] / m2[i]
-      }
-    }
-    
-    if (22 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m22[i] <- m20[i] / m13[i]
-      }
-    }
-    
-    if (23 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m23[i] <- m19[i] / m13[i]
-      }
-    }
-    
-    if (TRUE %in% (c(24:26) %in% measures)) {
-      EC <- c() #  early change
-      for (i in seq_len(nrow(data))) {
-        EC[i] <-
-          Last(data[i, 1:mid.position[i]]) - First(data[i, 1:mid.position[i]])
-      }
-      
-      LC <- c() #  later change
-      for (i in seq_len(nrow(data))) {
-        LC[i] <-
-          Last(data[i, mid.position[i]:ncol(data)]) - First(data[i, mid.position[i]:ncol(data)])
-      }
-    }
-    
-    if (24 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m24[i] <- EC[i] / LC[i]
-      }
-    }
-    
-    if (25 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m25[i] <- EC[i] / m5[i]
-      }
-    }
-    
-    if (26 %in% measures) {
-      for (i in seq_len(nrow(data))) {
-        output$m26[i] <- LC[i] / m5[i]
-      }
-    }
-    
-    output[is.na(output)] <-
-      1 #  NAs correspond to quotient measures of the form 0/0
-    
-    
-    # Remove the measures that are constant because (1) these are not useful for
-    # discriminating between the trajectories (2) they cause problem with the
-    # CheckCorrelation function later because their variance is 0 so division by 0
-    # occurs when computing correlation.
-    flag <- c()
-    for (j in seq_len(ncol(output))) {
-      col <- output[, j]
-      if (max(col) == min(col)) {
-        flag <- c(flag, j)
-      }
-    }
-    if (length(flag) > 0) {
-      output <- output[,-flag]
-      warning(paste(
-        "Measures ",
-        paste(colnames(output)[3:6], collapse = ", "),
-        " have been discarded due to being constant.",
-        sep = ""
-      ))
-    }
-    if (ncol(output) == 1) {
-      stop("All the measures are constant.")
-    }
     
     ######################################
     ##         Cap the outliers         ##
     ######################################
     
-    outliers <-
-      data.frame(matrix(nrow = nrow(output), ncol = ncol(output)))
-    colnames(outliers) <- colnames(output)
-    outliers[, 1] <- output$ID
+    if (cap.outliers == FALSE) {
+      outliers <- NULL
+    } 
     
-    for (j in 2:ncol(output)) {
-      y <- output[, j]
-      y.TRUE <- y
-      n <- length(y)
-      which.inf <- which(is.infinite(y.TRUE))
+    if (cap.outliers == TRUE){
+      outliers <-
+        data.frame(matrix(nrow = nrow(output), ncol = ncol(output)))
+      colnames(outliers) <- colnames(output)
+      outliers[, 1] <- output$ID
       
-      if (length(which.inf) > 0) {
-        y.TRUE <- y.TRUE[-which.inf]
-      }
-      
-      if (length(y.TRUE) > 2) {
-        top <-
-          rev(order(abs(y.TRUE - median(y.TRUE))))[1:ceiling(n * 0.01)] #  if n < 100, remove 1 element, so this is never empty
-        y.TRUE <- y.TRUE[-top]
-      }
-      
-      mu <- mean(y.TRUE)
-      sigma <- sd(y.TRUE)
-      
-      k_Cheb <-
-        sqrt(100 / 0.3) #  The classical Chebychev bound. Approximately 18.26.
-      k <- seq(from = 0.1, to = 18.26, by = 0.1)
-      M <- c()
-      for (i in seq(length(k))) {
-        max.left <-
-          max(density(
-            y.TRUE,
-            from = (mu - 18.3 * sigma),
-            to = (mu + 18.3 * sigma)
-          )$y[density(y.TRUE,
-                      from = (mu - 18.3 * sigma),
-                      to = (mu + 18.3 * sigma))$x > mu + k[i] * sigma])
-        max.right <-
-          max(density(
-            y.TRUE,
-            from = (mu - 18.3 * sigma),
-            to = (mu + 18.3 * sigma)
-          )$y[density(y.TRUE,
-                      from = (mu - 18.3 * sigma),
-                      to = (mu + 18.3 * sigma))$x < mu - k[i] * sigma])
-        M[i] <- max(max.left, max.right)
-      }
-      
-      p <- 2 * pi * k ^ 2 * (exp(1) - 2 * pi / 3)
-      q <-
-        2 * (2 * pi * k) ^ 3 / 27 - (2 * pi) ^ 2 * k ^ 3 * exp(1) / 3 - 2 * pi * exp(1) /
-        (sigma * M)
-      
-      root <-
-        CubeRoot(-q / 2 + sqrt(q ^ 2 / 4 + p ^ 3 / 27)) + CubeRoot(-q / 2 - sqrt(q ^
-                                                                                   2 / 4 + p ^ 3 / 27)) + 2 * pi * k / 3
-      
-      w <- which(root * sigma * M < 0.3 / 100)
-      if (length(w) > 0) {
-        k.opt <- min(k_Cheb, k[w[1]])
-      } else{
-        k.opt <- k_Cheb
-      }
-      
-      cap <- which(abs(y - mu) > k.opt * sigma)
-      if (length(cap) > 0) {
-        outliers[cap, j] <- signif(y[cap], 3)
+      for (j in 2:ncol(output)) {
+        y <- output[, j]
+        y.TRUE <- y
+        n <- length(y)
+        which.inf <- which(is.infinite(y.TRUE))
         
-        y[cap] <- mu + sign(y[cap]) * k.opt * sigma
-        output[, j] <- y
+        if (length(which.inf) > 0) {
+          y.TRUE <- y.TRUE[-which.inf]
+        }
+        
+        if (length(y.TRUE) > 2) {
+          top <-
+            rev(order(abs(y.TRUE - median(y.TRUE))))[1:ceiling(n * 0.01)] #  if n < 100, remove 1 element, so this is never empty
+          y.TRUE <- y.TRUE[-top]
+        }
+        
+        mu <- mean(y.TRUE)
+        sigma <- sd(y.TRUE)
+        
+        k_Cheb <-
+          sqrt(100 / 0.3) #  The classical Chebychev bound. Approximately 18.26.
+        k <- seq(from = 0.1, to = 18.26, by = 0.1)
+        M <- c()
+        for (i in seq(length(k))) {
+          max.left <-
+            max(density(
+              y.TRUE,
+              from = (mu - 18.3 * sigma),
+              to = (mu + 18.3 * sigma)
+            )$y[density(y.TRUE,
+                        from = (mu - 18.3 * sigma),
+                        to = (mu + 18.3 * sigma))$x > mu + k[i] * sigma])
+          max.right <-
+            max(density(
+              y.TRUE,
+              from = (mu - 18.3 * sigma),
+              to = (mu + 18.3 * sigma)
+            )$y[density(y.TRUE,
+                        from = (mu - 18.3 * sigma),
+                        to = (mu + 18.3 * sigma))$x < mu - k[i] * sigma])
+          M[i] <- max(max.left, max.right)
+        }
+        
+        p <- 2 * pi * k ^ 2 * (exp(1) - 2 * pi / 3)
+        q <-
+          2 * (2 * pi * k) ^ 3 / 27 - (2 * pi) ^ 2 * k ^ 3 * exp(1) / 3 - 2 * pi * exp(1) /
+          (sigma * M)
+        
+        root <-
+          CubeRoot(-q / 2 + sqrt(q ^ 2 / 4 + p ^ 3 / 27)) + CubeRoot(-q / 2 - sqrt(q ^
+                                                                                     2 / 4 + p ^ 3 / 27)) + 2 * pi * k / 3
+        
+        w <- which(root * sigma * M < 0.3 / 100)
+        if (length(w) > 0) {
+          k.opt <- min(k_Cheb, k[w[1]])
+        } else{
+          k.opt <- k_Cheb
+        }
+        
+        cap <- which(abs(y - mu) > k.opt * sigma)
+        if (length(cap) > 0) {
+          outliers[cap, j] <- signif(y[cap], 3)
+          
+          y[cap] <- mu + sign(y[cap]) * k.opt * sigma
+          output[, j] <- y
+        }
       }
+      
+      row.rm <- which(rowSums(!is.na(outliers[, -c(1), drop = FALSE])) == 0)
+      outliers <- outliers[-row.rm, , drop = FALSE]
+      col.kp <- which(colSums(!is.na(outliers)) != 0)
+      outliers <- outliers[, col.kp, drop = FALSE]
     }
-    
-    row.rm <- which(rowSums(!is.na(outliers[,-c(1)])) == 0)
-    outliers <- outliers[-row.rm,]
-    col.rm <- which(colSums(!is.na(outliers)) == 0)
-    outliers <- outliers[,-col.rm]
-    
+
     ID <- IDvector
     
     trajMeasures <-
@@ -739,7 +736,7 @@ print.trajMeasures <- function(x, ...) {
     print(x$measures)
   } else{
     measure.plus <- cbind(x$measures$ID, x$mid)
-    measure.plus <- cbind(measure.plus, x$measures[,-1])
+    measure.plus <- cbind(measure.plus, x$measures[, -1, drop = FALSE])
     colnames(measure.plus)[1:2] <- c("ID", "mid time")
     print(measure.plus, row.names = FALSE)
   }
@@ -749,37 +746,28 @@ print.trajMeasures <- function(x, ...) {
 #'
 #'@export
 summary.trajMeasures <- function(object, ...) {
-  cat("Description of the measures:\n\n")
-  cat("m1: Range\n")
-  cat("m2: Mean of the trajectory\n")
-  cat("m3: Standard deviation (SD)\n")
-  cat("m4: Coefficient of variation (m3/m2)\n")
-  cat("m5: Overall change (initial value - final value)\n")
-  cat("m6: Mean change per unit time\n")
-  cat("m7: Overall change relative to initial value\n")
-  cat("m8: Overall change relative to mean (m5/m2)\n")
-  cat("m9: Slope of the linear model\n")
-  cat("m10: Proportion of variance explained by the linear model (R squared)\n")
-  cat("m11: Maximum value of the speed\n")
-  cat("m12: Functional SD of the speed\n")
-  cat("m13: Mean absolute speed\n")
-  cat("m14: Maximum absolute speed\n")
-  cat("m15: Maximum absolute speed relative to the mean (m14/m2)\n")
-  cat("m16: Maximum absolute speed relative to the slope (m14/m9)\n")
-  cat("m17: Functional SD of the speed relative to the slope (m12/m9)\n")
-  cat("m18: Mean acceleration\n")
-  cat("m19: Mean absolute acceleration\n")
-  cat("m20: Maximum of the absolute acceleration\n")
-  cat("m21: Maximum of the absolute acceleration relative to the mean (m20/m2)\n")
-  cat(
-    "m22: Maximum of the absolute acceleration relative to the mean absolute speed (m20/m13)\n"
-  )
-  cat("m23: Mean absolute acceleration relative to the mean absolute speed (m19/m13)\n")
-  cat("m24: Early change relative to later change\n")
-  cat("m25: Early change relative to overall change\n")
-  cat("m26: Later change relative to overall change\n\n")
   
+  cat("Description of the measures:\n")
+  cat("m1: Maximum\n")
+  cat("m2: Range\n")
+  cat("m3: Mean value\n")
+  cat("m4: Standard deviation\n")
+  cat("m5: Slope of the linear model\n")
+  cat("m6: Proportion of variance explained by the linear model (R squared)\n")
+  cat("m7: Curve length (total variation)\n")
+  cat("m8: Number of times crossing the mean per unit time\n")
+  cat("m9: Proportion of time spent under the mean\n")
+  cat("m10: Minimum of the first derivative\n")
+  cat("m11: Maximum of the first derivative\n")
+  cat("m12: Mean of the first derivative\n")
+  cat("m13: Standard deviation of the first derivative\n")
+  cat("m14: Minimum of the second derivative\n")
+  cat("m15: Maximum of the second derivative\n")
+  cat("m16: Mean of the second derivative\n")
+  cat("m17: Standard deviation of the second derivative\n")
+  cat("m18: Early change/Later change\n")
   
+  cat("\n")
   
   cat("Summary of measures:\n")
   
@@ -809,28 +797,32 @@ summary.trajMeasures <- function(object, ...) {
   measures.summary[6,] <- apply(object$measures, 2, max)[-1]
   
   print(measures.summary)
-  cat("\n")
   
-  cat("Outliers before capping:\n")
-  outliers <- object$outliers
-  outliers.pre <- outliers
-  outliers.pre[is.na(outliers.pre)] <- ""
-  print(outliers.pre, row.names = FALSE)
   
-  cat("Outliers after capping:\n")
-  if (nrow(outliers) == 0) {
-    print(outliers, row.names = FALSE)
-  } else{
-    outliers.post <- outliers
-    for (j in seq_len(nrow(outliers))) {
-      for (k in 2:ncol(outliers)) {
-        if (is.na(outliers[j, k]) == FALSE) {
-          outliers.post[j, k] <-
-            signif(object$measures[object$measures$ID == outliers$ID[j], colnames(outliers)[k]], 3)
+  if(!is.null(object$outliers)){
+    cat("\n")
+    
+    cat("Outliers before capping:\n")
+    outliers <- object$outliers
+    outliers.pre <- outliers
+    outliers.pre[is.na(outliers.pre)] <- ""
+    print(outliers.pre, row.names = FALSE)
+    
+    cat("Outliers after capping:\n")
+    if (nrow(outliers) == 0) {
+      print(outliers, row.names = FALSE)
+    } else{
+      outliers.post <- outliers
+      for (j in seq_len(nrow(outliers))) {
+        for (k in 2:ncol(outliers)) {
+          if (is.na(outliers[j, k]) == FALSE) {
+            outliers.post[j, k] <-
+              signif(object$measures[object$measures$ID == outliers$ID[j], colnames(outliers)[k]], 3)
+          }
         }
       }
+      outliers.post[is.na(outliers.post)] <- ""
+      print(outliers.post, row.names = FALSE)
     }
-    outliers.post[is.na(outliers.post)] <- ""
-    print(outliers.post, row.names = FALSE)
   }
 }
